import base64
import hashlib
import os
from flask import request
from srht.database import DbSession
from srht.config import cfg, get_origin
from srht.flask import SrhtFlask, csrf_bypass
from srht.oauth import current_token, oauth
from legacyshim.oauth import LegacyOAuthService

db = DbSession(cfg("legacy.sr.ht", "connection-string"))
db.init()

from legacyshim.types import User, Upload

class LegacyApp(SrhtFlask):
    def __init__(self):
        super().__init__("legacy.sr.ht", __name__,
                oauth_service=LegacyOAuthService())

app = LegacyApp()

def file_link(path):
    return f"{get_origin('legacy.sr.ht')}/{path}"

def extension(f):
    parts = f.rsplit('.', 2)
    # special case for .tar.* files
    if len(parts) > 2 and parts[-2].lower() == 'tar':
        return 'tar.' + parts[-1].lower()
    return parts[-1].lower()

@app.route("/api/upload", methods=["POST"])
@oauth("uploads:write")
@csrf_bypass
def uploads_POST():
    if not current_token.user.upload_permitted:
        return { "errors": [
            { "reason": "Your account is not whitelisted for this service" }
        ] }, 401
    f = request.files.get('file')
    filename = ''.join(c for c in f.filename if c.isalnum() or c == '.')
    upload = Upload()
    upload.user_id = current_token.user_id
    upload.hash = get_hash(f)
    existing = Upload.query.filter(Upload.hash == upload.hash).first()
    if existing:
        return {
            "success": True,
            "hash": existing.hash,
            "shorthash": existing.shorthash,
            "url": file_link(existing.path)
        }
    len = 4
    shorthash = upload.hash[:len]
    while any(Upload.query.filter(Upload.shorthash == shorthash)):
        len += 1
        shorthash = upload.hash[:len]
    upload.shorthash = shorthash
    upload.path = os.path.join(upload.shorthash + "." + extension(filename))
    upload.original_name = f.filename

    f.seek(0)
    f.save(os.path.join(cfg("legacy.sr.ht", "storage"), upload.path))

    if not upload.shorthash:
        return {
            "success": False,
            "error": "Upload interrupted"
        }

    db.session.add(upload)
    db.session.commit()
    return {
        "success": True,
        "hash": upload.hash,
        "shorthash": upload.shorthash,
        "url": file_link(upload.path),
    }

def get_hash(f):
    f.seek(0)
    return base64.urlsafe_b64encode(hashlib.md5(f.read()).digest()).decode()

@app.route("/api/uploads")
@oauth("uploads:read")
def uploads_GET():
    user_id = current_token.user_id
    uploads = (Upload.query
            .filter(Upload.user_id == user_id, Upload.hidden == False)
            .order_by(Upload.created.desc()))
    return [u.to_dict() for u in uploads]
