import sqlalchemy as sa
from datetime import datetime, timezone
from srht.database import Base
from srht.oauth import ExternalUserMixin, ExternalOAuthTokenMixin

class OAuthToken(Base, ExternalOAuthTokenMixin):
    pass

class User(Base, ExternalUserMixin):
    upload_permitted = sa.Column(sa.Boolean, nullable=False, server_default='f')

class Upload(Base):
    __tablename__ = 'upload'
    id = sa.Column(sa.Integer, primary_key=True)
    user_id = sa.Column(sa.Integer, sa.ForeignKey('user.id'))
    user = sa.orm.relationship('User',
            backref=sa.orm.backref('upload', order_by=id, lazy='dynamic'))
    hash = sa.Column(sa.String, nullable=False)
    shorthash = sa.Column(sa.String, nullable=False)
    path = sa.Column(sa.String, nullable=False)
    created = sa.Column(sa.DateTime(timezone=True))
    original_name = sa.Column(sa.Unicode(512))
    hidden = sa.Column(sa.Boolean)

    def __init__(self):
        self.created = datetime.now(timezone.utc)
        self.hidden = False

    def to_dict(self):
        return {
            'hash': self.hash,
            'shorthash': self.shorthash,
            'path': self.path,
            'original_name': self.original_name,
            'created': self.created.strftime('%FT%T%Z'),
        }
